/*
 * HammingMatchingAutomaton.h
 *
 *  Created on: 19. 3. 2018
 *      Author: Tomas Capek
 */

#ifndef _GENERALIZED_LEVENSHTEIN_MATCHING_AUTOMATON_H__
#define _GENERALIZED_LEVENSHTEIN_MATCHING_AUTOMATON_H__

#include <automaton/FSM/EpsilonNFA.h>
#include <string/LinearString.h>
#include <stringology/matching/LevenshteinMatchingAutomaton.h>


namespace stringology {

namespace matching {

class GeneralizedLevenshteinMatchingAutomaton {
public:
	/**
	 * Creates Levenshtein matching automata.
	 *
	 * @return automata for aproximate string matching using Hamming algorithm
	 */
	template < class SymbolType >
	static automaton::EpsilonNFA < SymbolType, void, ext::pair<unsigned int, unsigned int> > construct(const string::LinearString < SymbolType > & pattern, unsigned int allowed_errors);
};


template < class SymbolType >
automaton::EpsilonNFA < SymbolType, void, ext::pair<unsigned int, unsigned int> > GeneralizedLevenshteinMatchingAutomaton::construct(const string::LinearString < SymbolType > & pattern, unsigned int allowed_errors) {
	automaton::EpsilonNFA < SymbolType, void, ext::pair<unsigned int, unsigned int> > result = stringology::matching::LevenshteinMatchingAutomaton::construct(pattern, allowed_errors);

	for (unsigned int j = 0; j<allowed_errors; j++) {
		for (unsigned int i = j; i + 1 < pattern.getContent().size(); i++) {
			auto from = ext::make_pair(i, j);
			auto transpose_state = ext::make_pair(pattern.getContent().size()+1+i, j);
			auto to = ext::make_pair(i+2, j+1);

			result.addState(transpose_state);
			result.addTransition(from, pattern.getContent()[i+1], transpose_state);
			result.addTransition(transpose_state, pattern.getContent()[i], to);
		}
	}

	return result;
}


} /* namespace matching */

} /* namespace stringology */

#endif /* _HAMMING_MATCHING_AUTOMATON_H__ */
